# Fachschemareferenz
## Zielsetzung
### Wie ist eine Einreichung aufgebaut?

![Struktur einer Einreichung](../../static/images/schema_reference/submission_structure.svg)

Eine Einreichung über FIT-Connect besteht aus einem obligatorischen Metadatensatz, der die Einreichung beschreibt.
Der Metadatensatz gibt an, wie viele Anlagen beigefügt werden.
Zusätzlich zu den allgemeinen Anlagen kann ein Fachdatensatz im JSON- oder XML-Format beigefügt werden.
Im Zustellpunkt wird definiert, ob für die gewählte Verwaltungsleistung ein Fachdatensatz beigefügt wird und welchem Format er entsprechen muss.

### Warum sollte eine Fachschemareferenz angegeben werden?

Empfangende Stellen in FIT-Connect definieren im Zustellpunkt Schemata für Einreichungen wie Anträge oder Berichte, die diese Stellen im Empfang akzeptieren.
Durch die Angabe des verwendeten Fachschemas in den Metadaten der Einreichung signalisiert das sendende System, welches der angegeben Schemata verwendet wird und erlaubt eine schnelle und einfache Validierung auf der Empfangsseite.
Durch die standardisierte Nutzung von Fachschemareferenzen in den Metadaten der Einreichung und dem Zustellpunkt der Empfangsstelle wird eine Interoperabilität über viele unterschiedliche Fachstandards und Fachdatenformate hinweg gewährleistet.

## Technische Beschreibung
### Aufbau Fachschemareferenz

Eine Fachschemareferenz besteht aus zwei Angaben.
- Die `schemaURI` referenziert das zu verwendende Fachschema.
- Der `mimeType` gibt den Datentyp (JSON oder XML) der Fachdaten vor.

Bitte beachten Sie, dass manche Fachschema nur JSON oder XML als Datentyp zulassen.

### FIM Fachschemareferenz

Um ein FIM Fachschema eindeutig zu referenzieren sind folgende Angaben notwendig:

- **ID** des Stammdatenschemas mit kleinem "S" ("s00000000009" im Beispiel)
- **Version** des Stammdatenschemas ("1.0.0" im Beispiel)
- **Art** des Schemas (JSON oder XML Schema)

Für ein JSON Schema wird die schemaURI wird wie folgt gebildet:
`https://schema.fitko.de/fim/{ID}_{Version}.schema.json`

Für ein XML Schema wird die schemaURI wird wie folgt gebildet:
`https://schema.fitko.de/fim/{ID}_{Version}.xsd`

#### Beispiel für eine FIM Fachschemareferenz

JSON Schema:
```json
  {
    "schemaURI": "https://schema.fitko.de/fim/s00000000009_1.0.0.schema.json",
    "mimeType": "application/json"
  }
```

XML Schema:
```json
  {
    "schemaURI": "https://schema.fitko.de/fim/s00000000009_1.0.0.xsd",
    "mimeType": "application/xml"
  }
```

### XÖV Fachschemareferenz

Ein XÖV Fachschema wird über folgende Angaben referenziert:
- **Kennung** (ID) des Fachschemas ("urn:xoev-de:bmk:standard:xbau" im Beispiel)
- **Version** des Fachschemas ("2.2" im Beispiel)
- Verwendete **Nachricht** aus dem Fachschema ("baugenehmigung.antrag.0200" im Beispiel)

Beispiel: `urn:xoev-de:bmk:standard:xbau_2.2#baugenehmigung.antrag.0200`

Die schemaURI wird wie folgt gebildet:
`{Kennung}_{Version}#{Nachricht}`

#### Beispiel für eine XÖV Fachschemareferenz

```json
  {
    "schemaURI": "urn:xoev-de:bmk:standard:xbau_2.2#baugenehmigung.antrag.0200",
     "mimeType": "application/xml"
  }
```

### Fachschemareferenz(en) im Zustellpunkt

Der Zustellpunkt enthält ein Array "schemas", das eine Liste von Fachschemareferenzen enthält.

```json
  "schemas": [
    {
      "schemaURI": "https://schema.fitko.de/fim/s00000000009_1.0.0.schema.json",
      "mimeType": "application/json"
    },
    {
      "schemaURI": "https://schema.fitko.de/fim/s00000000009_1.1.0.schema.json",
      "mimeType": "application/json"
    }
    {
      "schemaURI": "https://schema.fitko.de/fim/s00000000010_1.0.0.schema.json",
      "mimeType": "application/json"
    },
    {
      "schemaURI": "https://schema.fitko.de/fim/s00000000010_1.1.0.schema.json",
      "mimeType": "application/json"
    }
  ],
```

### Fachschemareferenz im Metadatensatz

Sofern der Zustellpunkt mindestens eine Fachschemareferenz enthält, muss der Metadatensatz genau eine davon enthalten.

```json
  "contentStructure": {
    "data": {
      "schema": {
        "schemaURI": "https://schema.fitko.de/fim/s00000000009_1.0.0.schema.json",
        "mimeType": "application/json"
      }
    },
```

## Anwendung

Im folgenden wird das oben beschriebene Zusammenspiel einmal in der üblichen zeitlichen Reihenfolge durchgespielt.
Es wird vorausgesetzt, dass bereits ein Fachschema vorliegt.
Weiterführende Informationen enthält der [OZG-Leitfaden](https://leitfaden.ozg-umsetzung.de/) im Kapitel [8.6 Föderales Informationsmanagement (FIM)](https://leitfaden.ozg-umsetzung.de/pages/viewpage.action?pageId=4621502).

### Zustellpunkt anlegen

Beim [Anlegen des Zustellpunktes](../apis/delivery-service#post-/destinations) wird eine Liste von Fachschemareferenzen angegeben.
Diese wird i.d.R. aus genau einem Eintrag bestehen.
Sofern mehrere unterschiedliche Verfahren über einen Zustellpunkt abgewickelt werden, kann die Liste auch mehrere Fachschemareferenzen enthalten.

```http
POST /destinations
```

```json
{
  "contactInformation": {
    "legalName": "Max",
    "address": "Musterstr. 31, 10000 Berlin, Deutschland",
    "phone": "+49170123456789",
    "email": "max@mustermann.not",
    "unit": "Department XYZ"
  },
  "schemas": [
    {
      "schemaURI": "https://schema.fitko.de/fim/s00000000009_1.0.0.schema.json",
      "mimeType": "application/json"
    },
    {
      "schemaURI": "urn:xoev-de:bmk:standard:xbau_2.2#baugenehmigung.antrag.0200",
      "mimeType": "application/xml"
    }
  ],
  "callback": "https://www.example.com/webhook/beispiel",
  "encryptionKid": "c66e4423-e28d-4a1f-911d-818f9ab60221",
  "publicKeys": {
    "keys": [
      {
        "kty": "RSA",
        "key_ops": [
          "verify",
          "wrapKey"
        ],
        "alg": "RSA-OAEP-256",
        "x5c": [
          "...(base64 encoded cert)...",
          "...(base64 encoded intermediate cert)...",
          "...(base64 encoded root cert)..."
        ],
        "kid": "c66e4423-e28d-4a1f-911d-818f9ab60221",
        "n": "sS_HZU9sYgMFhqv5fyCn8Y1dQ91lO8JfsWsG3OwREKgfRyso1brjcYO2aT1EWdGF6L7836R9otyIHJLhPOfn-D6pV8LYeoSnsDNqU8QdGR3yyDhfaZprWJkBDXfMMlpSJdNFZWeQ0oqEHcd37yowKYPBEE8UqTKMYAuhZ6YuouH8zjM2VoyYtZJMjja4ZcgXoPesBFkVUdzr3HpD2wJjvXXqLFROsw4HpAiFgR6iccmMb7i93sG5dro2XS-0RjWYbO2RcP5ClJD7-s0kvhO7_KAXDkacVvRV275UWttjDNAok9FHvFgBIzYAMNBGKxPFZxEaT3J5FM9XIzsk6AcqJw",
        "e": "AQAB"
      }
    ]
  }
}
```

### Zustellpunkt abfragen

Das antragsendende System [fragt den Zustellpunkt ab](../apis/delivery-service#get-/destinations/-destinationId-) und erhält einen Teil der hinterlegten Informationen.
Diese enthalten die Liste der Fachschemareferenzen.

```http
GET /destinations/{destinationId}
```

```json
{
  "schemas": [
    {
      "schemaURI": "https://schema.fitko.de/fim/s00000000009_1.0.0.schema.json",
      "mimeType": "application/json"
    },
    {
      "schemaURI": "urn:xoev-de:bmk:standard:xbau_2.2#baugenehmigung.antrag.0200",
      "mimeType": "application/xml"
    }
  ],
  "encryptionKid": "c66e4423-e28d-4a1f-911d-818f9ab60221",
  "publicKeys": {
    "keys": [
      {
        "kty": "RSA",
        "key_ops": [
          "verify",
          "wrapKey"
        ],
        "alg": "RSA-OAEP-256",
        "x5c": [
          "...(base64 encoded cert)...",
          "...(base64 encoded intermediate cert)...",
          "...(base64 encoded root cert)..."
        ],
        "kid": "c66e4423-e28d-4a1f-911d-818f9ab60221",
        "n": "sS_HZU9sYgMFhqv5fyCn8Y1dQ91lO8JfsWsG3OwREKgfRyso1brjcYO2aT1EWdGF6L7836R9otyIHJLhPOfn-D6pV8LYeoSnsDNqU8QdGR3yyDhfaZprWJkBDXfMMlpSJdNFZWeQ0oqEHcd37yowKYPBEE8UqTKMYAuhZ6YuouH8zjM2VoyYtZJMjja4ZcgXoPesBFkVUdzr3HpD2wJjvXXqLFROsw4HpAiFgR6iccmMb7i93sG5dro2XS-0RjWYbO2RcP5ClJD7-s0kvhO7_KAXDkacVvRV275UWttjDNAok9FHvFgBIzYAMNBGKxPFZxEaT3J5FM9XIzsk6AcqJw",
        "e": "AQAB"
      }
    ]
  }
}
```

### Fachschema aussuchen

Aus der Liste der Fachschemareferenzen muss das antragsendende System eine aussuchen.

### Metdadatensatz generieren

Im Metadatensatz wird die ausgewählte Fachschemareferenz unter `contentStructure.data.schema` benannt.

```json
{
  "contentStructure": {
    "data": {
      "schema": {
        "schemaURI": "https://schema.fitko.de/fim/s00000000009_1.0.0.schema.json",
        "mimeType": "application/json"
      }
    },
    "attachments": [
      {
        "attachmentId": "8f482392-43ad-418c-9185-30e4e350e76b",
        "purpose": "form",
        "size": 13046,
        "mimeType": "application/pdf",
        "filename": "test.pdf",
        "description": "Das Antragsformular"
      }
    ]
  }
}
```

### Fachdatensatz generieren

Es muss ein zur Referenz passender Fachdatensatz generiert und [mitgesendet](../apis/delivery-service#post-/submissions/-submissionId-) werden.

```json
{
  "F00000152": "Kirchheim bei München",
  "G00000246": {
    "F00000457": "091840131131"
  },
  "G00000091": [
    {
      "F60000319": "Beispiel GmbH",
      "F00000103": "350"
    }
  ]
}
```

### Einreichung empfangen

Der Empfänger [holt die Einreichung ab](../apis/delivery-service#get-/submissions/-submissionId-).

```http
GET /submissions/{submissionId}
```

Diese enthält unter anderem den verschlüsselten Metadaten- sowie Fachdatensatz.

```json
{
  "destinationId": "879ee109-a690-4db8-ab32-424284184d7d",
  "submissionId": "ce75a6b8-d72f-4b94-b09e-af6be35bc2ae",
  "attachments": [
    "879ee109-a690-4db8-ab32-424284184d7d",
    "2046f9f1-dc89-4440-9c24-c76a8f40d668"
  ],
  "encryptedMetadata": "eyJlbmMiOiJBMjU2R0NNIiwiYWxnIjoiUlNBLU9BRVAtMjU2In0.nlXGAufYH36IABDy0En0LXEhGfC20IZSSchs27ADalHpRoTZKfXhc7hcMk8Y9V8yTP0jYbmrq6NtEg-QS2O5TQFD9Hluhpb631PBgKjPXHYX1Y6iUcR1sXxSUPjePi8F8PcZUZuUJLnhz6myyc9scdAq9BXG2cDJVgkfLI8eZdrqnrY24Hh32_7d5OKLFSpSDrBlqfyQuY8Wbs2h8Wy4Z4hwT1aWDO7b-SqJA181hUbNcF_rR4Mze3Fdtu-3uOIQYgLBBRmN1ZHDLk0EKNCI4B8MyDKLGPoM0ZomV5lVwVWjAMRI4CgQkIQ9rnm-Adof-GbegQL3yJSoNIWRWgzCnZBYZ638QgPllCMVW3WvEVvsgj0Hj16PbofqXTQ5S73LINfP6FZawfC0yMrYjSV_N2L0Lkp2KI3BkJcy-PcFhBnhwu2IsJGAlyDRCnXdVqig8m5yLHuSMQTpLW69LzPEskfsjhnNDR-CEBZpicjMfc-4CL6U7E7YoGc_99DzE5U5._JfqyKH23GiKsnDW.ZtMMjZ3GgcgHss8qbFRhrjl4L0kAfbco-oXICkk.VBHJ00FyDTYjOA_OYfiz5g",
  "encryptedData": "eyJlbmMiOiJBMjU2R0NNIiwiYWxnIjoiUlNBLU9BRVAtMjU2In0.nlXGAufYH36IABDy0En0LXEhGfC20IZSSchs27ADalHpRoTZKfXhc7hcMk8Y9V8yTP0jYbmrq6NtEg-QS2O5TQFD9Hluhpb631PBgKjPXHYX1Y6iUcR1sXxSUPjePi8F8PcZUZuUJLnhz6myyc9scdAq9BXG2cDJVgkfLI8eZdrqnrY24Hh32_7d5OKLFSpSDrBlqfyQuY8Wbs2h8Wy4Z4hwT1aWDO7b-SqJA181hUbNcF_rR4Mze3Fdtu-3uOIQYgLBBRmN1ZHDLk0EKNCI4B8MyDKLGPoM0ZomV5lVwVWjAMRI4CgQkIQ9rnm-Adof-GbegQL3yJSoNIWRWgzCnZBYZ638QgPllCMVW3WvEVvsgj0Hj16PbofqXTQ5S73LINfP6FZawfC0yMrYjSV_N2L0Lkp2KI3BkJcy-PcFhBnhwu2IsJGAlyDRCnXdVqig8m5yLHuSMQTpLW69LzPEskfsjhnNDR-CEBZpicjMfc-4CL6U7E7YoGc_99DzE5U5._JfqyKH23GiKsnDW.ZtMMjZ3GgcgHss8qbFRhrjl4L0kAfbco-oXICkk.VBHJ00FyDTYjOA_OYfiz5g",
  "currentStatus": "queued",
  "statusHistory": [
    {
      "sourceState": "incomplete",
      "targetState": "queued",
      "timestamp": "2021-01-30T08:30:00Z"
    }
  ]
}
```

### Metadatensatz prüfen

Der Empfänger [entschlüsselt mit seinem privaten Schlüssel](../getting-started/receiving/decrypt.mdx) den Metadatensatz.
Nach der Entschlüsselung des Metadatensatzes prüft der Empfänger, ob eine Fachdatenreferenz vorhanden ist und diese einer der im Zustellpunkt definierten Fachschemareferenz entspricht.
Sofern dies nicht der Fall ist, [wird die Einreichung zurückgewiesen](../getting-started/event-log.mdx#events) (siehe Event `https://schema.fitko.de/fit-connect/events/reject-submission`).

### Fachdatensatz prüfen

Nach erfolgter Prüfung des Metadatensatzes entschlüsselt der Empfänger den Fachdatensatz.
Danach erfolgt eine Prüfung des Fachdatensatzes:
- Entspricht der Fachdatensatz dem vereinbarten Medientyp (JSON oder XML)?
- Entspricht der Fachdatensatz dem vereinbarten Schema?

**Hinweis:** Dieses Dokument beschreibt nur den Ablauf und die Prüfungen, die sich auf die Fachdatenreferenz beziehen. Beim Empfang von Übertragungen sind noch weitere Prüfungen durchzuführen.

### Details zur Prüfung eines FIM Fachdatensatzes (JSON Schema)

:::caution
Das FITKO Schemarepository ist derzeit noch in Vorbereitung.
Daher können die Fachschemata noch nicht wie beschrieben heruntergeladen werden.
:::

Die FIM-Fachschemata werden im FITKO Schemarepository (`https://schema.fitko.de/`) abgelegt.
Die Fachschemareferenz gibt an, welches Schema zu beziehen ist.

```json
  "contentStructure": {
    "data": {
      "schema": {
        "schemaURI": "https://schema.fitko.de/fim/s00000000009_1.0.0.schema.json",
        "mimeType": "application/json"
      }
```

- Prüfen Sie, dass die `schemaURI` mit `https://schema.fitko.de/fim/` beginnt. Dies stellt sicher, dass das Fachschema aus dem FITKO Repository bezogen wird.
- Laden Sie das Fachschema herunter

```
➜ wget -nv https://schema.fitko.de/fim/s00000000009_1.0.0.schema.json
```

- Nun können Sie den Fachdatensatz gegen das vorliegende Fachschema "s00000000009_1.0.0.schema.json" prüfen.

### Details zur Prüfung eines FIM Fachdatensatzes (XML Schema)

Die Fachschemata von XFall können über das FIM-Portal bezogen werden.
Zukünftig werden sie auch über das FITKO Schemarepository zur Verfügung stehen.

```json
  "contentStructure": {
    "data": {
      "schema": {
        "schemaURI": "https://schema.fitko.de/fim/s00000092_1.0.xsd",
        "mimeType": "application/xml"
     }
```

- Prüfen Sie, dass die `schemaURI` mit `https://schema.fitko.de/fim/` beginnt.
- Entnehmen Sie der URI die ID des Fachschemas (hier: "s00000092").
- Öffnen Sie das FIM-Portal (https://fimportal.de/) und loggen Sie sich ein.
- Öffnen Sie Sie Adresse `https://fimportal.de/detail/D/{ID}`, also z.B. https://fimportal.de/detail/D/S00000092.
- Klicken Sie rechts auf den Downloadlink, der auf "_xfall.xsd" endet (hier: `S00000092V1.0_xfall.xsd`).
- Das XFall-Schema wird Ihnen angezeigt und kann abgespeichert werden.
- Nun können Sie den Fachdatensatz gegen das vorliegende Fachschema "S00000092V1.0_xfall.xsd" prüfen.

### Details zur Prüfung eines XÖV Fachdatensatzes

Die XÖV Standards werden im [XRepository](https://www.xrepository.de/) abgelegt.
Auf die Inhalte des XRepository kann über eine [REST-API](https://www.xrepository.de/cms/hilfe.html#rest-api) zugegriffen werden.
Entnehmen Sie der Fachschemareferenz die Angabe `schemaURI`. Diese besteht aus zwei durch ein Doppelkreuz (#) getrennten Teilen. Vor dem Doppelkreuz steht die Kennung, dahinter der Name der Nachricht.
Um z.B. einen Bauantrag zu prüfen, gehen Sie wie folgt vor.
- Im Metadatensatz finden Sie folgende Fachschemareferenz.

```json
  "contentStructure": {
    "data": {
      "schema": {
        "schemaURI": "urn:xoev-de:bmk:standard:xbau_2.2#baugenehmigung.antrag.0200",
        "mimeType": "application/xml"
     }
```

- Die `schemaURI` enthält folgende Angaben:
  - Kennung: `urn:xoev-de:bmk:standard:xbau_2.2`
  - Nachrichtenname `baugenehmigung.antrag.0200`
- Laden Sie das XML Schema herunter und entpacken Sie es.

```
➜ wget -nv -O xmlschema.zip https://www.xrepository.de/api/version_standard/urn:xoev-de:bmk:standard:xbau_2.2/xmlschema
2021-06-25 15:17:54 URL:https://www.xrepository.de/api/version_standard/urn:xoev-de:bmk:standard:xbau_2.2/xmlschema [278585] -> "xmlschema.zip" [1]
```

```
➜ unzip xmlschema.zip
Archive:  xmlschema.zip
  inflating: xsd.zip
  inflating: xsd_dev.zip
```

- Im Falle von XBau 2.2 enthält das XML-Schema zwei Dateien. Enpacken Sie die Datei `xsd.zip`. Dieser Schritt ist XBau-spezifisch und muss bei anderen Standards angepasst werden.

```
➜ unzip xsd.zip
Archive:  xsd.zip
  inflating: xbau-basisdatentypen.xsd
  inflating: xbau-baukasten.xsd
  inflating: xbau-codes.xsd
  inflating: xbau-nachrichten-abweichung.xsd
  inflating: xbau-nachrichten-anzeige.xsd
  inflating: xbau-nachrichten-baugenehmigung.xsd
  inflating: xbau-nachrichten-baulasten.xsd
  inflating: xbau-nachrichten-bautechnischeNachweise.xsd
  inflating: xbau-nachrichten-bauzustand.xsd
  inflating: xbau-nachrichten-beteiligung.xsd
  inflating: xbau-nachrichten-genehmigungsfreistellung.xsd
  inflating: xbau-nachrichten-informationsempfaenger.xsd
  inflating: xbau-nachrichten-kammernverzeichnis.xsd
  inflating: xbau-nachrichten-teilbaugenehmigung.xsd
  inflating: xbau-nachrichten-ueberwachungspflichtigeAnlagen.xsd
  inflating: xbau-nachrichten-vorbescheid.xsd
  inflating: xbau-prozessnachrichten.xsd
```

- Suchen Sie nun nach der Datei, die die Elementdefinition der geforderten Nachricht enthält.

```
➜ grep -l 'name=.baugenehmigung.antrag.0200' *.xsd
xbau-nachrichten-baugenehmigung.xsd
```

- Nun können Sie den Fachdatensatz gegen das vorliegende Fachschema "xbau-nachrichten-baugenehmigung.xsd" prüfen.
